/* 
 * This file is part of XCmail and is subject to the 
 * XCmail license agreement
 */

/* $Id: plugins.h,v 2.9 2001/10/10 17:12:02 schmitzj Exp $ */


#ifndef PLUGINS_H
#define PLUGINS_H

#include <Xclasses/Xclasses.h>

/*---------------------------------------------------------------------------
 * General plugins:
 *  abstract class, only PLF (plugin flags) interesting:
 *   PLF_STATICUSE    - the one object if this class is used for all work
 *   PLF_CLONEUSE     - the object is cloned before use
 *   PLF_CONFIGURABLE - plugin has config options
 *   PLF_INVISIBLE    - don't show plugin in settings window
 */

#define PLF_STATICUSE		0x00000000
#define PLF_CLONEUSE		(1<<0)		// clone object before use
#define PLF_CONFIGURABLE	(1<<1)		// plugin has config options (makeconfig, readconfig, readconfigfile, writeconfigfile defined)
#define PLF_INVISIBLE		(1<<2)

#define PLF_USERFLAGSSTART	10

class plugins : public callable, public node	// abstract
{
	char *name;
friend class pluginmgr;
	char *author;
	char *information;
	char *filename;
	group *configgp;

	void plugin_filename(char *);
public:
	enum PLUGINTYPE { PT_None, PT_Encoder, PT_Helper, PT_Maintool, PT_Showmailtool, PT_Newmailtool, PT_Filterspec, PT_SCondition, PT_SJob };

	enum PLUGINTYPE type;
	unsigned int flags;

	// global contructor
	plugins(char *name,char *author,enum PLUGINTYPE t=PT_None,unsigned int flags=0,char *information=NULL);
	// clone contructor
	plugins(class plugins *);
	virtual ~plugins();

	char *plugin_name(void);
	char *plugin_author(void);
	char *plugin_information(void);
	char *plugin_filename(void);
	char *programversion(void);
	char *settingspath(void);		// full path of ~/.XCmail/

	char *egets(char *,int,char **buf);
	void eputs(char *,char **buf);

	virtual class plugins *clone(void);

	virtual group *makeconfig(void);
	virtual void setupconfig(group *);
	virtual void readconfig(group *);

	virtual void readconfigfile(void);
	virtual void writeconfigfile(void);

	int newmail(char *to,char *cc,char *subject,char *body);	// 0: error, 1: sent, 2: async sending
	int replymail(class mail *,char *subject,char *body);		// 0: error, 1: sent, 2: async sending
	int forwardmail(class mail *,char *to,char *subject);		// 0: error, 1: sent, 2: async sending

	class folders *folderlist(void);
	class iolist *iolist(void);
};

/*---------------------------------------------------------------------------
 * Main window plugins:
 *  all plugins for the main window belong to this,
 *  the existing object is called with init() when the mainwindow
 *  opens, finish() is called before it is closed (but not for clones!)
 *  durring init() the addmenu function may be called with one of its options:
 *   (MPMENU flags,ID,Menutext,Popupmenutext)
 *   MPMENU_GENERAL   - this item may be called every time (handle(int))
 *   MPMENU_SELECTED  - this item may only be called if a mail is selected (handle(int,mail*))
 *   MPMENU_MARKED    - this item may only be called if mail is selected or marked (handle(int,mail*) for all marked mail or the selected ones)
 *
 *  If PLF_CLONEUSE is set the object is cloned before used (clone()->handle()),
 *  the object must free itself!
 *
 *  Popupmenutext is currently not supported!
 */

#define MPMENU_GENERAL		(1<<0)
#define MPMENU_SELECTED		(1<<1)
#define MPMENU_MARKED		(1<<2)

class mainplugin : public plugins
{
public:
	virtual class mainplugin *clone(void);
	mainplugin(char *name,char *author,unsigned int flags=0,char *information=NULL);
	mainplugin(class mainplugin *);
	~mainplugin();

	void addmenu(unsigned int flags,int id,char *text,char *popuptext=NULL,char *shortcut=NULL);

	virtual void init(void);
	virtual void finish(void);
	virtual void handle(int id,class mail *);
	virtual void handle(int id);
};

/*---------------------------------------------------------------------------
 * Newmail window plugins:
 *  all plugins for the newmail window belong to this,
 *  the existing object is called with init() when a newmail window
 *  opens, finish() is called before it is closed (for every window, for every clone!)
 *  durring init() the addmenu function may be called with one of its options
 *
 *  If PLF_CLONEUSE is set the object is cloned before used (durring newmail window opening).
 *  and freed when the window is closed.
 *
 */

class newmailplugin : public plugins
{
public:
	virtual class newmailplugin *clone(void);
	newmailplugin(char *name,char *author,unsigned int flags=0,char *information=NULL);
	newmailplugin(class newmailplugin *);
	~newmailplugin();

	void addmenu(int id,char *text,char *shortcut=NULL);

	virtual void init(class Xclasses *);			// class newmail
	virtual void finish(class Xclasses *);			// class newmail
	virtual void handle(int id,class Xclasses *);	// class newmail
};

/*---------------------------------------------------------------------------
 * Showmail window plugins:
 *  all plugins for the showmail window belong to this,
 *  the existing object is called with init() when a showmail window
 *  opens, finish() is called before it is closed (for every window, for every clone!)
 *  durring init() the addmenu function may be called with one of its options
 *
 *  If PLF_CLONEUSE is set the object is cloned before used (durring showmail window opening),
 *  and freed when the window is closed.
 *
 */

class showmailplugin : public plugins
{
public:
	virtual class showmailplugin *clone(void);
	showmailplugin(char *name,char *author,unsigned int flags=0,char *information=NULL);
	showmailplugin(class showmailplugin *);
	~showmailplugin();

	void addmenu(int id,char *text,char *shortcut=NULL);

	virtual void init(class Xclasses *);			// class showmail
	virtual void finish(class Xclasses *);			// class showmail
	virtual void handle(int id,class Xclasses *);	// class showmail
};

/*---------------------------------------------------------------------------*/

#endif
